
# **********************************************************************;
# * Project           : xPI.blocks
# *
# * Program name      : HCSR04.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200417
# *
# * Purpose           : Handle with HC-SR04 ultrasonic distance sensor
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202004    Viacheslav Karpizin      1       original version
# *
#
# Copyright (c) 2019, Al Audet , 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.sparkfun.com/datasheets/Robotics/TB6612FNG.pdf
# **********************************************************************
import RPi.GPIO as GPIO
import time
import smbus_m
import sys
sys.path.append('./../')


GPIO.setmode(GPIO.BOARD)
# GPIO.setmode(GPIO.BCM)


class HCSR04:

	# Constructor
	def __init__(self, pin_transmit, pin_receive):

		self._trig = pin_transmit
		self._echo = pin_receive

		GPIO.setup(self._trig, GPIO.OUT)
		GPIO.setup(self._echo, GPIO.IN)

	def __del__(self):
		GPIO.cleanup()



    def __init__(
        self, trig_pin, echo_pin, temperature=20, unit="metric", gpio_mode=GPIO.BCM
    ):
        self.trig_pin = trig_pin
        self.echo_pin = echo_pin


    def raw_distance(self, sample_size=11, sample_wait=0.1):
        """Return an error corrected unrounded distance, in cm, of an object 
        adjusted for temperature in Celcius.  The distance calculated
        is the median value of a sample of `sample_size` readings.
        Speed of readings is a result of two variables.  The sample_size
        per reading and the sample_wait (interval between individual samples).
        Example: To use a sample size of 5 instead of 11 will increase the 
        speed of your reading but could increase variance in readings;
        value = sensor.Measurement(trig_pin, echo_pin)
        r = value.raw_distance(sample_size=5)
        Adjusting the interval between individual samples can also
        increase the speed of the reading.  Increasing the speed will also
        increase CPU usage.  Setting it too low will cause errors.  A default
        of sample_wait=0.1 is a good balance between speed and minimizing 
        CPU usage.  It is also a safe setting that should not cause errors.
        e.g.
        r = value.raw_distance(sample_wait=0.03)
        """

 

        speed_of_sound = 331.3 * math.sqrt(1 + (self.temperature / 273.15))
        sample = []
        # setup input/output pins
        GPIO.setwarnings(False)

        GPIO.setup(self.trig_pin, GPIO.OUT)
        GPIO.setup(self.echo_pin, GPIO.IN)

        for distance_reading in range(sample_size):
            GPIO.output(self.trig_pin, GPIO.LOW)
            time.sleep(sample_wait)
            GPIO.output(self.trig_pin, True)
            time.sleep(0.00001)
            GPIO.output(self.trig_pin, False)
            echo_status_counter = 1
            while GPIO.input(self.echo_pin) == 0:
                if echo_status_counter < 1000:
                    sonar_signal_off = time.time()
                    echo_status_counter += 1
                else:
                    raise SystemError("Echo pulse was not received")
            while GPIO.input(self.echo_pin) == 1:
                sonar_signal_on = time.time()
            time_passed = sonar_signal_on - sonar_signal_off
            distance_cm = time_passed * ((speed_of_sound * 100) / 2)
            sample.append(distance_cm)
        sorted_sample = sorted(sample)
        # Only cleanup the pins used to prevent clobbering
        # any others in use by the program
        GPIO.cleanup((self.trig_pin, self.echo_pin))
        return sorted_sample[sample_size // 2]

    def depth(self, median_reading, hole_depth):
        """Calculate the depth of a liquid. hole_depth is the
        distance from the sensor to the bottom of the hole."""

        return hole_depth - median_reading

    def distance(self, median_reading):
        """Calculate the distance from the sensor to an object."""
        return median_reading



    @staticmethod
    def basic_distance(trig_pin, echo_pin, celsius=20):
        """Return an unformatted distance in cm's as read directly from
        RPi.GPIO."""

        speed_of_sound = 331.3 * math.sqrt(1 + (celsius / 273.15))
        GPIO.setup(trig_pin, GPIO.OUT)
        GPIO.setup(echo_pin, GPIO.IN)
        GPIO.output(trig_pin, GPIO.LOW)
        time.sleep(0.1)
        GPIO.output(trig_pin, True)
        time.sleep(0.00001)
        GPIO.output(trig_pin, False)
        echo_status_counter = 1
        while GPIO.input(echo_pin) == 0:
            if echo_status_counter < 1000:
                sonar_signal_off = time.time()
                echo_status_counter += 1
            else:
                raise SystemError("Echo pulse was not received")
        while GPIO.input(echo_pin) == 1:
            sonar_signal_on = time.time()

        time_passed = sonar_signal_on - sonar_signal_off
        return time_passed * ((speed_of_sound * 100) / 2)






    
    
def main():
	print "=========================================================="
	print "HC-SR04 demo for xPI.blocks "
	print "=========================================================="
	time.sleep(0.5)
	
  test = HCSR04(pin1, pin2)

	try:
		while True:


			time.sleep(2)
			
	except KeyboardInterrupt:
		# Ctrl-C to exit
		sys.exit(0)


if __name__ =="__main__":
    main()   
    



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : max17040g.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200117
# *
# * Purpose           : Read battery parameters from 3.7 lithium battery (used in UPS Lite module for RPI Zero)
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202001    Viacheslav Karpizin      1      original version
# *
#
# Copyright (c) 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************
import time
import smbus_m
import sys
sys.path.append('./../')


class MAX17040G:

    def __init__(self, bus_id, addr):
        """ Define connection
        Parameters:
        - addr: 0 = address pin is low; 1 = address pin is high.
        - bus:  The SMBus used. Rev 1 Pi uses 0; Rev 2 Pi uses 1.
        - mode: 0 to 5 = the mode as defined in the order above
        """
        self.bus_id = bus_id
        self.bus = smbus_m.SMBus3(self.bus_id)
        self.addr = addr

    def readVoltage(self):
        "This function returns as float the voltage from the Raspi UPS Hat via the provided SMBus object"
        read = self.bus.read_word_data(self.addr, 2)
        swapped = (read >> 8) + ((read & 0xFF) << 8)
        voltage = swapped*1.25/1000/16
        return voltage

    def readCapacity(self):
        "This function returns as a float the remaining capacity of the battery connected to the Raspi UPS Hat via the provided SMBus object"
        read = self.bus.read_word_data(self.addr, 4)
        swapped = (read >> 8) + ((read & 0xFF) << 8)
        capacity = swapped/256.0
        return capacity


def main():
    print "=========================================================="
    print "MAX17040G demo for RPI series"
    print "=========================================================="
    time.sleep(0.5)

    bus = 1

    address = 0x36        # MAX17040G
    sensor = MAX17040G(bus, address)

    try:
        while True:
            print("Capacity: %5.2f %", sensor.readCapacity())
            print("Voltage: %5.2f V", sensor.readVoltage())
            time.sleep(10)
    except KeyboardInterrupt:
        # Ctrl-C to exit
        sys.exit(0)


if __name__ == "__main__":
    main()



from bme280 import BME280
from bh1750 import BH1750
import time
import sys
from ads1115 import ADS1115
sys.path.append('./env/')


def main():
    print("===================== weather demo script ====================")

    time.sleep(1)

    bus = 0
    adc_addr = 0x48
    light_addr = 0x23
    bmp280_addr = 0x76

    a = ADS1115(bus, adc_addr)

    l = BH1750(bus, light_addr)

    b = BME280(bus, bmp280_addr)

    counter = 0

    try:
        while True:
            counter = counter + 1
            time.sleep(1)
            adc_0 = a.readAdc(0)
            adc_1 = a.readAdc(1)
            adc_2 = a.readAdc(2)
            adc_3 = a.readAdc(3)

            light = l.getLight()

            temp = b.read_temperature()
            pressure = b.read_pressure()
            humidity = b.read_humidity()

            print("adc: " + adc_0 + " " + adc_1 + " " + adc_2 + " " + adc_3 + " light: " +
                  light + " temp :" + temp + " pressure: " + pressure + " hum: " + humidity)

    except KeyboardInterrupt:
        print("Interrupt from keyboard")
        sys.exit(0)


if __name__ == "__main__":
    main()



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : pcf8574.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20191119
# *
# * Purpose           : Handle 8-channel I2C in/out multiplexer
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 201911    Viacheslav Karpizin      1      original version
# *
#
# Copyright (c) 2019, Viacheslav Karpizin
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


import smbus_m
import time
import sys

sys.path.append('./../')
sys.path.append('./')


class PCF8574:
    def __init__(self, bus_id, address):
        self.bus_no = bus_id
        self.bus = smbus_m.SMBus3(self.bus_no)
        self.address = address

    def write(self, pin, value):
        """
        Set a specific output high (True) or low (False).
        """
        assert pin in range(
            8), "Output number must be an integer between 0 and 7"
        current_state = self.bus.read_byte(self.address)
        bit = 1 << 7-pin
        new_state = current_state | bit if value else current_state & (
            ~bit & 0xff)
        self.bus.write_byte(self.address, new_state)

    def read(self, pin):
        """
        Get the boolean state of an individual pin.
        """
        assert pin in range(8), "Pin number must be an integer between 0 and 7"
        state = self.bus.read_byte(self.address)
        return bool(state & 1 << 7-pin_number)


def main():

    print "=========================================================="
    print "PCF8574 demo for RPI series"
    print "=========================================================="
    time.sleep(0.5)

    addr = 0x20

    bus = 1
    chip = PCF8574(bus, addr)

    while True:
        chip.write(0, 1)
        time.sleep(1)
        chip.write(1, 1)
        time.sleep(1)
        chip.write(2, 1)
        time.sleep(1)
        chip.write(3, 1)
        time.sleep(1)
        chip.write(4, 1)
        time.sleep(1)
        chip.write(5, 1)
        time.sleep(1)
        chip.write(6, 1)
        time.sleep(1)
        chip.write(7, 1)
        time.sleep(1)

        chip.write(0, 0)
        time.sleep(1)
        chip.write(1, 0)
        time.sleep(1)
        chip.write(2, 0)
        time.sleep(1)
        chip.write(3, 0)
        time.sleep(1)
        chip.write(4, 0)
        time.sleep(1)
        chip.write(5, 0)
        time.sleep(1)
        chip.write(6, 0)
        time.sleep(1)
        chip.write(7, 0)
        time.sleep(1)


if __name__ == "__main__":
    main()





# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : x9cxxx.py
# *
# * Author            : Viacheslav Karpizin,
# *
# * Date created      : 20200315
# *
# * Purpose           : Controlling digital potentiometer (x9c102, x9c103, x9c104, x9c503)
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1      original version
# *

#
# Copyright (c) 2019, Viacheslav Karpizin, (c) 2016 Adafruit Industries, Tony DiCola
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.renesas.com/eu/en/www/doc/datasheet/x9c102-103-104-503.pdf
# **********************************************************************


import time
import RPi.GPIO as GPIO
import random


GPIO.setmode(GPIO.BOARD)

#GPIO.setup(channel, GPIO.OUT)
# GPIO.input(channel)
#GPIO.output(channel, state)


DIGIPOT_UP = 1
DIGIPOT_DOWN = 0
DIGIPOT_MAX_AMOUNT = 99
DIGIPOT_UNKNOWN = 255


class X9Cxxx:

    def __init__(self, incPin, udPin, csPin):
        self._incPin = incPin
        self._udPin = udPin
        self._csPin = csPin
        self._currentValue = DIGIPOT_UNKNOWN

        GPIO.setup(self._incPin, GPIO.OUT)
        GPIO.setup(self._udPin, GPIO.OUT)
        GPIO.setup(self._csPin, GPIO.OUT)
        GPIO.output(self._csPin, GPIO.HIGH)

    def reset(self):
        print("inside reset")
        self.decrease(DIGIPOT_MAX_AMOUNT)
        self._currentValue = 0
        print(" currentValue = " + str(self._currentValue))

    def set(self, value):
        value = max(0, value)
        value = min(value, DIGIPOT_MAX_AMOUNT)

        if (self._currentValue == DIGIPOT_UNKNOWN):
            print("reset")
            self.reset()

        if (self._currentValue > value):
            self.change(DIGIPOT_DOWN, self._currentValue-value)
            print("going down for " + str(self._currentValue-value))
        elif (self._currentValue < value):
            self.change(DIGIPOT_UP, value-self._currentValue)
            print("going up for " + str(value-self._currentValue))

    def get(self):
        return _currentvalue

    def increase(self, amount):
        amount = max(0, amount)
        amount = min(amount, DIGIPOT_MAX_AMOUNT)
        self.change(DIGIPOT_UP, amount)

    def decrease(self, amount):
        print("inside decrease")
        amount = self.constrain(amount,  0, DIGIPOT_MAX_AMOUNT)
        self.change(DIGIPOT_DOWN, amount)

    def constrain(self, val, min_val, max_val):
        return min(max_val, max(min_val, val))

    def change(self, direction, amount):
        print("inside change")
        print("currentValue= " + str(self._currentValue))
        print("direction, amount " + str(direction) + " " + str(amount))

        amount = self.constrain(amount,  0, DIGIPOT_MAX_AMOUNT)

        GPIO.output(self._udPin, direction)
        GPIO.output(self._incPin, GPIO.HIGH)
        GPIO.output(self._csPin, GPIO.LOW)

        for i in range(0, amount):

            GPIO.output(self._incPin, GPIO.LOW)
            time.sleep(0.0002)

            GPIO.output(self._incPin, GPIO.HIGH)
            time.sleep(0.0002)
            if (self._currentValue != DIGIPOT_UNKNOWN):
                # _currentValue += (direction == DIGIPOT_UP ? 1 : -1)
                if direction == DIGIPOT_UP:
                    self._currentValue += 1
                else:
                    self._currentValue -= 1
                self._currentValue = self.constrain(
                    self._currentValue, 0, DIGIPOT_MAX_AMOUNT)

        GPIO.output(self._csPin, GPIO.HIGH)

        print(" (end) currentValue= " + str(self._currentValue))


def main():
    print "=========================================================="
    print "X9Cxxx demo for RPI series"
    print "=========================================================="
    time.sleep(2)

    #inc = 38
    #ud = 40
    #cs = 37

    pot = X9Cxxx(38, 40, 37)

    try:
        while True:
            value = 5  # = random.randint(0, 99)

            print("value = " + str(value))
            pot.set(value)

            time.sleep(5)

            value = 95  # = random.randint(0, 99)
            print("value = " + str(value))
            pot.set(value)

            time.sleep(5)

            value = 5  # = random.randint(0, 99)
            print("value = " + str(value))
            pot.set(value)

            time.sleep(5)

            value = 95  # = random.randint(0, 99)
            print("value = " + str(value))
            pot.set(value)

            time.sleep(5)

            value = 5  # = random.randint(0, 99)
            print("value = " + str(value))
            pot.set(value)

            time.sleep(5)

    except KeyboardInterrupt:
        # Ctrl-C to exit
        sys.exit(0)


if __name__ == "__main__":
    main()



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : VNH2SP30.py
# *
# * Author            : Viacheslav Karpizin,
# *
# * Date created      : 20200315
# *
# * Purpose           : Controlling powerful motor driver VNH2SP30
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1      original version
# *

#
# Copyright (c) 2019, Viacheslav Karpizin, (c) 2016 Adafruit Industries, Tony DiCola
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# **********************************************************************


import time
import RPi.GPIO as GPIO
import random


GPIO.setmode(GPIO.BOARD)


DIGIPOT_UP = 1
DIGIPOT_DOWN = 0
DIGIPOT_MAX_AMOUNT = 99
DIGIPOT_UNKNOWN = 255


class VNH2SP30:

  ina = ""
	inb = ""
	pwm = ""

	# Defaults
	freq = 200

	  def __init__(self, pwm, ina, inb):
      self._pwm = pwm
      self._ina = ina
      self._inb = inb  



      GPIO.setup(self._pwm, GPIO.OUT)		
      GPIO.setup(self._ina, GPIO.OUT)		
      GPIO.setup(self._inb, GPIO.OUT)
      GPIO.output(self._ina, GPIO.HIGH)
      GPIO.output(self._inb, GPIO.HIGH)

      self.pa = GPIO.PWM(self._pwm, self.freq)

      self.pa.start(0)


	  def set(self, power):
      print("inside set")
      //power:-100 to 100
      if power > 100:
        power = 100
      elif power < -100:
        power = -100

      if abs(power) > 0:
              if power > 0:
                      GPIO.output(self._ina, GPIO.HIGH)
                      GPIO.output(self._inb, GPIO.LOW)
              else:
                      GPIO.output(self._ina, GPIO.LOW)
                      GPIO.output(self._inb, GPIO.HIGH)

      else:
                      GPIO.output(self._ina, GPIO.HIGH)
                      GPIO.output(self._inb, GPIO.HIGH)


      self.pa.start(abs(power)/100)


	








def main():
	print "=========================================================="
	print "VNH2SP30 demo for RPI series"
	print "=========================================================="
	time.sleep(2)

	# pwm = 40
	# ina = 38
	# inb = 37


	motor = VNH2SP30(40, 38, 37)



	try:
		while True:
                for  i in range(-100, 100):
                        motor.turnMotor(i)
                        time.sleep(0.01)
                for  i in range(100, -100):
                        motor.turnMotor(i)
                        time.sleep(0.01)
			


			time.sleep(5)			


	except KeyboardInterrupt:
		# Ctrl-C to exit
		sys.exit(0)


if __name__ =="__main__":
    main()



#!/usr/bin/env python


# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : ads1115.py
# *
# * Author            : Viacheslav Karpizin,
# *
# * Date created      : 20191017
# *
# * Purpose           : Read data from 4-channel ADC
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 201910    Viacheslav Karpizin      1      original version
# *
# # ADS115 Datasheet can be found here:
# http://www.ti.com/lit/ds/symlink/ads1113.pdf
#
# Copyright (c) 2019, Viacheslav Karpizin, D. Scott Williamson
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


# Imports
#
from __future__ import absolute_import
import time  # used for sleep function
import sys   # used for print function
#from . import smbus_m   # I2Cimport 
#from . import smbus_m
#import smbus_m
#import .smbus_m
#from .smbus_m import SMBus3
import smbus_m

class ADS1115:

    ################################################################################
    #
    # Variables
    #

    # get access to the SMBus (SMBus is a subset of the I2C protocol
    # 0 = /dev/i2c-0 (port I2C0)
    # 1 = /dev/i2c-1 (port I2C1) *this is the port on the GPIO's of RPi 3 B
    # bus = smbus_m.SMBus3(1)    # the 1 means use port I2C1

    # Address for the device

    # addr=0x48       # ADDR tied to GND
    # addr=0x49       # ADDR tied to VDD
    # addr=0x4A       # ADDR tied to SDA
    # addr=0x4B       # ADDR tied to SCL

    # Registers in the ADS1115
    DEVICE_REG_CONVERSION = 0x00
    DEVICE_REG_CONFIG = 0x01
    DEVICE_REG_LO_THRESH = 0x02
    DEVICE_REG_HI_THRESH = 0x03

    # Configuration register fields

    # Operational Status
    CONFIG_OS = 0X8000
    CONFIG_OS_START = 0X8000
    CONFIG_OS_PERFORMING_CONVERSION = 0X0000
    CONFIG_OS_NOT_PERFORMING_OPERATION = 0X8000

    # Differential measurements
    CONFIG_MUX_AIN0P_AIN1N = 0X0000  # (default)
    CONFIG_MUX_AIN1P_AIN3N = 0X1000
    CONFIG_MUX_AIN2P_AIN3N = 0X2000
    CONFIG_MUX_AIN3P_AIN3N = 0X3000
    #  Single ended measurements
    CONFIG_MUX_AIN0P_GNDN = 0X4000
    CONFIG_MUX_AIN1P_GNDN = 0X5000
    CONFIG_MUX_AIN2P_GNDN = 0X6000
    CONFIG_MUX_AIN3P_GNDN = 0X7000

    # Programmable gain amplifier configuration
    CONFIG_FSR_6V144 = 0X0000
    CONFIG_FSR_4V096 = 0X0200
    CONFIG_FSR_2V048 = 0X0400  # (default)
    CONFIG_FSR_1V024 = 0X0600
    CONFIG_FSR_0V512 = 0X0800
    CONFIG_FSR_0V256 = 0X0A00
    CONFIG_FSR_0V256 = 0X0C00
    CONFIG_FSR_0V256 = 0X0E00

    # Continuous or single shot mode
    CONFIG_MODE_CONTINUOUS = 0X0000
    CONFIG_MODE_SINGLE_SHOT = 0X0100  # (default)

    # Data rate
    CONFIG_DATA_RATE_8SPS = 0X0000
    CONFIG_DATA_RATE_16SPS = 0X0020
    CONFIG_DATA_RATE_32SPS = 0X0040
    CONFIG_DATA_RATE_64SPS = 0X0060
    CONFIG_DATA_RATE_128SPS = 0X0080  # (default)
    CONFIG_DATA_RATE_2508SPS = 0X00A0
    CONFIG_DATA_RATE_475SPS = 0X00C0
    CONFIG_DATA_RATE_860SPS = 0X00E0

    # Comparitor mode
    CONFIG_COMP_MODE_TRADITIONAL = 0X0000  # (default)
    CONFIG_COMP_MODE_WINDOW = 0X0010

    # Comparitor polarity
    CONFIG_COMP_POL_ACTIVE_LOW = 0X0000  # (default)
    CONFIG_COMP_POL_ACTIVE_HIGH = 0X0008

    # Comparitor latching
    CONFIG_COMP_LAT = 0X0004
    CONFIG_COMP_LAT_NON_LATCHING = 0X0000  # (default)
    CONFIG_COMP_LAT_LATCHING = 0X0004

    # comparitor queue and disable
    CONFIG_COMP_QUE = 0X0003
    CONFIG_COMP_QUE_1_CONV = 0X0000
    CONFIG_COMP_QUE_2_CONV = 0X0001
    CONFIG_COMP_QUE_4_CONV = 0X0002
    CONFIG_COMP_QUE_DISABLE = 0X0003  # (default)

################################################################################
#
# Functions

    def __init__(self, busid, addr,
                 voltage=CONFIG_FSR_4V096,
                 freq=CONFIG_DATA_RATE_128SPS):

        assert(addr == 0x48 or addr == 0x49 or addr == 0x4A or addr == 0x4B)
        assert(voltage == self.CONFIG_FSR_4V096
               or voltage == self.CONFIG_FSR_6V144
               or voltage == self.CONFIG_FSR_2V048
               or voltage == self.CONFIG_FSR_1V024)

        self._bus = smbus_m.SMBus3(busid)
        self._addr = addr
        self._voltage = voltage
        self._freq = freq

    def getVoltLimit(self):
        return self._voltage

    # swap endian of word (swap high and low bytes)
    # This is needed because the communication protocol uses a different byte
    # order than the Raspberry Pi
    def swap(self, a):
        return ((a & 0xff00) >> 8) | ((a & 0x00ff) << 8)

    # read ADC channel (blocking but quick)
    # all of the hardware access is in this function
    def readAdc(self, channel):
        #print("reading value...")
        # sanity check the channel specified
        if ((channel > 3) or (channel < 0)):
            return -1

        # Build read command
        config = (self.CONFIG_OS_START +				# start conversion
                  self.CONFIG_MUX_AIN0P_GNDN + 		# single ended conversion
                  (channel << 12) + 				# select channel
                  self._voltage + 				# 4.096v pre amp (3.3v signal)
                  self.CONFIG_MODE_SINGLE_SHOT + 		# single conversion and shutdown
                  self._freq + 		# data rate
                  self.CONFIG_COMP_MODE_TRADITIONAL + 	# comp conventional
                  self.CONFIG_COMP_POL_ACTIVE_LOW + 	# comp active low
                  self.CONFIG_COMP_LAT_NON_LATCHING + 	# comp non latching
                  self.CONFIG_COMP_QUE_DISABLE)		# comp disabled

        # Uncomment to see the config command in printed in hexadecimal
        # print 'config: %04X' % config

        # send read command (note byte swap)
        self._bus.write_word_data(
            self._addr, self.DEVICE_REG_CONFIG, self.swap(config))

        # wait for conversion to complete (blocking)
        while (True):
            # read status (note byte swap)
            status = self.swap(self._bus.read_word_data(
                self._addr, self.DEVICE_REG_CONFIG))

            # Uncomment to see the status printed in hexadecimal
            # print 'status: %04X' % status

            # when the Operational Status is no longer performing a conversion
            # we can break out of this wait loop
            if (status & self.CONFIG_OS) != self.CONFIG_OS_PERFORMING_CONVERSION:
                break

        # read result (note byte swap)
        result = self.swap(self._bus.read_word_data(
            self._addr, self.DEVICE_REG_CONVERSION))

        # return 16 bit integer A2D result for the specified channel
        return result


def main():
    print("==========================================================")
    print("ADS1115 demo for RPI series")
    print("==========================================================")
    time.sleep(1)

    bus = 0
    addr = 0x48

    a = ADS1115(bus, addr)

    print("sensor initialized...")
    try:
        while True:
            s = "(Ctrl-C to exit) ADC Result: "
            for i in range(0, 4):
                val = a.readAdc(i)
                s += str(i)+": %05d" % val+"  "
            print(s)
            time.sleep(0.5)
    except KeyboardInterrupt:

        print("interruption")
        # Ctrl-C to exit
        sys.exit(0)


if __name__ == "__main__":
    main()




# **********************************************************************;
# * Project           : xPI.blocks
# *
# * Program name      : DS1302.py
# *
# * Author            : Viacheslav Karpizin, L.Lefebvre
# *
# * Date created      : 20200418
# *
# * Purpose           : Driver for DS1302 clock chip (GPIO)
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202004    Viacheslav Karpizin      1      original version
# *
#
#

#
# Copyright (c) 2018, https://github.com/sourceperl, L.Lefebvre; (c) 2020, Viacheslav Karpizin
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


import time
import datetime
import RPi.GPIO as GPIO
import binascii
import datetime


class DS1302:
    # 5us
    CLK_DELAY = 5E-6

    def __init__(self, clk_pin=11, data_pin=13, ce_pin=15):
        # init GPIO
        # no warnings
        GPIO.setwarnings(False)
        # use safer pin number (avoid GPIO renumber on each Pi release)
        GPIO.setmode(GPIO.BOARD)
        # set GPIO pins
        self._clk_pin = clk_pin
        self._data_pin = data_pin
        self._ce_pin = ce_pin
        # CLK and CE (sometime call RST) pin are always output
        GPIO.setup(self._clk_pin, GPIO.OUT, initial=GPIO.LOW)
        GPIO.setup(self._ce_pin, GPIO.OUT, initial=GPIO.LOW)
        # turn off WP (write protect)
        self._start_tx()
        self._w_byte(0x8e)
        self._w_byte(0x00)
        self._end_tx()
        # charge mode is disabled
        self._start_tx()
        self._w_byte(0x90)
        self._w_byte(0x00)
        self._end_tx()

    def _start_tx(self):
        """
        Start of transaction.
        """
        GPIO.output(self._clk_pin, GPIO.LOW)
        GPIO.output(self._ce_pin, GPIO.HIGH)

    def _end_tx(self):
        """
        End of transaction.
        """
        GPIO.setup(self._data_pin, GPIO.IN)
        GPIO.output(self._clk_pin, GPIO.LOW)
        GPIO.output(self._ce_pin, GPIO.LOW)

    def _r_byte(self):
        """
        Read byte from the chip.
        :return: byte value
        :rtype: int
        """
        # data pin is now input (pull-down resistor embedded in chip)
        GPIO.setup(self._data_pin, GPIO.IN)
        # clock the byte from chip
        byte = 0
        for i in range(8):
            # make a high pulse on CLK pin
            GPIO.output(self._clk_pin, GPIO.HIGH)
            time.sleep(self.CLK_DELAY)
            GPIO.output(self._clk_pin, GPIO.LOW)
            time.sleep(self.CLK_DELAY)
            # chip out data on clk falling edge: store current bit into byte
            bit = GPIO.input(self._data_pin)
            byte |= ((2 ** i) * bit)
        # return byte value
        return byte

    def _w_byte(self, byte):
        """
        Write byte to the chip.
        :param byte: byte value
        :type byte: int
        """
        # data pin is now output
        GPIO.setup(self._data_pin, GPIO.OUT)
        # clock the byte to chip
        for _ in range(8):
            GPIO.output(self._clk_pin, GPIO.LOW)
            time.sleep(self.CLK_DELAY)
            # chip read data on clk rising edge
            GPIO.output(self._data_pin, byte & 0x01)
            byte >>= 1
            GPIO.output(self._clk_pin, GPIO.HIGH)
            time.sleep(self.CLK_DELAY)

    def read_ram(self):
        """
        Read RAM as bytes
        :return: RAM dumps
        :rtype: bytearray
        """
        # start of message
        self._start_tx()
        # read ram burst
        self._w_byte(0xff)
        # read data bytes
        byte_a = bytearray()
        for _ in range(31):
            byte_a.append(self._r_byte())
        # end of message
        self._end_tx()
        return byte_a

    def write_ram(self, byte_a):
        """
        Write RAM with bytes
        :param byte_a: bytes to write
        :type byte_a: bytearray
        """
        # start message
        self._start_tx()
        # write ram burst
        self._w_byte(0xfe)
        # write data bytes
        for i in range(min(len(byte_a), 31)):
            self._w_byte(ord(byte_a[i:i + 1]))
        # end of message
        self._end_tx()

    def read_datetime(self):
        """
        Read current date and time from RTC chip.
        :return: date and time
        :rtype: datetime.datetime
        """
        # start message
        self._start_tx()
        # read clock burst
        self._w_byte(0xbf)
        byte_l = []
        for _ in range(7):
            byte_l.append(self._r_byte())
        # end of message
        self._end_tx()
        # decode bytes
        second = ((byte_l[0] & 0x70) >> 4) * 10 + (byte_l[0] & 0x0f)
        minute = ((byte_l[1] & 0x70) >> 4) * 10 + (byte_l[1] & 0x0f)
        hour = ((byte_l[2] & 0x30) >> 4) * 10 + (byte_l[2] & 0x0f)
        day = ((byte_l[3] & 0x30) >> 4) * 10 + (byte_l[3] & 0x0f)
        month = ((byte_l[4] & 0x10) >> 4) * 10 + (byte_l[4] & 0x0f)
        year = ((byte_l[6] & 0xf0) >> 4) * 10 + (byte_l[6] & 0x0f) + 2000
        # return datetime value
        return datetime.datetime(year, month, day, hour, minute, second)

    def write_datetime(self, dt):
        """
        Write a python datetime to RTC chip.
        :param dt: datetime to write
        :type dt: datetime.datetime
        """
        # format message
        byte_l = [0] * 9
        byte_l[0] = (dt.second // 10) << 4 | dt.second % 10
        byte_l[1] = (dt.minute // 10) << 4 | dt.minute % 10
        byte_l[2] = (dt.hour // 10) << 4 | dt.hour % 10
        byte_l[3] = (dt.day // 10) << 4 | dt.day % 10
        byte_l[4] = (dt.month // 10) << 4 | dt.month % 10
        byte_l[5] = (dt.weekday() // 10) << 4 | dt.weekday() % 10
        byte_l[6] = ((dt.year-2000) // 10) << 4 | (dt.year-2000) % 10
        # start message
        self._start_tx()
        # write clock burst
        self._w_byte(0xbe)
        # write all data
        for byte in byte_l:
            self._w_byte(byte)
        # end of message
        self._end_tx()

    @staticmethod
    def close():
        """
        Clean all GPIOs.
        """
        GPIO.cleanup()


def main():
	print "=========================================================="
	print "DS1302 demo for xPI.blocks"
	print "=========================================================="
	time.sleep(1)

    rtc = DS1302()

    try:
        # read date and time from RTC chip
        print(rtc.read_datetime())

        # write current UTC date and time to chip
        rtc.write_datetime(datetime.datetime.utcnow())

        # read RAM and print dump
        print(binascii.hexlify(rtc.read_ram()))

        # write ram
        rtc.write_ram(b'\xDE\xAD\xBE\xEF')
    finally:
        # clean close
        rtc.close()   

if __name__ =="__main__":
    main()  


     



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : ShiftRegister74HC595.py
# *
# * Author            : Viacheslav Karpizin,
# *
# * Date created      : 20200405
# *
# * Purpose           : Output shift register (serial to parallel)
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202004    Viacheslav Karpizin      1      original version
# *

#
# Copyright (c) 2020, Viacheslav Karpizin,
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


import RPi.GPIO as GPIO
from time import sleep

GPIO.setmode(GPIO.BCM)


Class HC595():

    # Define MODES
    ALL = -1
    HIGH = 1
    LOW = 0

    # is used to store states of all pins
    _registers = list()

    # How many of the shift registers - you can change them with shiftRegisters method
    _number_of_shiftregisters = 1

       # ser_pin - serial, data in
       # rclk_pin - latch
       # srclk_pin - clock
       def __init__(self,  ser_pin, rclk_pin, srclk_pin, number_of_bits=8):

            custompins = 0
            self._serpin = SER_pin
            self._rclkpin = RCLK_pin
            self._srclkpin = SRCLK_pin
            self._number_of_bits = number_of_bits

            GPIO.setwarnings(True)

            GPIO.setup(self._serpin, GPIO.OUT)
            GPIO.setup(self._rclkpin, GPIO.OUT)
            GPIO.setup(self._srclkpin, GPIO.OUT)

        def startupMode(mode, execute= False):
            '''
            Allows the user to change the default state of the shift registers outputs
            '''
            if isinstance(mode, int):
                if mode is HIGH or mode is LOW:
                    _all(mode, execute)
                else:
                    raise ValueError(
                        "The mode can be only HIGH or LOW or Dictionary with specific pins and modes")
            elif isinstance(mode, dict):
                for pin, mode in mode.iteritems():
                    _setPin(pin, mode)
                if execute:
                    _execute()
            else:
                raise ValueError(
                    "The mode can be only HIGH or LOW or Dictionary with specific pins and modes")


        def digitalWrite(pin, mode):
            '''
            Allows the user to set the state of a pin on the shift register
            '''
            if pin == ALL:
                _all(mode)
            else:
                if len(_registers) == 0:
                    _all(LOW)

                _setPin(pin, mode)
            _execute()

        def delay(millis):
            '''
            Used for creating a delay between commands
            '''
            millis_to_seconds = float(millis)/1000
            return sleep(millis_to_seconds)

        def _all_pins():
            return _number_of_shiftregisters * 8

        def _all(mode, execute= True):

            #all_shr = _all_pins()

            for pin in range(0, self._number_of_bits):
                _setPin(pin, mode)
            if execute:
                _execute()

            return _registers

        def _setPin(pin, mode):
            try:
                _registers[pin] = mode
            except IndexError:
                _registers.insert(pin, mode)

        def _execute():
            #all_pins = _all_pins()
            GPIO.output(_RCLK_pin, GPIO.LOW)

            for pin in range(self._number_of_bits - 1, -1, -1):
                GPIO.output(_SRCLK_pin, GPIO.LOW)

                pin_mode = _registers[pin]

                GPIO.output(_SER_pin, pin_mode)
                GPIO.output(_SRCLK_pin, GPIO.HIGH)

            GPIO.output(_RCLK_pin, GPIO.HIGH)


def main():
    print "=========================================================="
    print "74HC595 shift register (serial in, parallel out) demo for RPI series"
    print "=========================================================="
    time.sleep(1)

    c = HC595(37, 38, 40)


    print("chip initialized...")
    try:
        while True:
            c.digitalWrite(1, GPIO.LOW)
            c.digitalWrite(2, GPIO.HIGH)


            time.sleep(0.5)
    except KeyboardInterrupt:

        print("interruption")
        # Ctrl-C to exit
        sys.exit(0)

if __name__ == "__main__":
    main()




# TODO: add serial operations


import smbus_m
import time
import sys
import ctypes

c_uint8 = ctypes.c_uint8
c_uint16 = ctypes.c_uint16
c_uint32 = ctypes.c_uint32


class Flag_bits(ctypes.Structure):
    _fields_ = [("rangeNumber", c_uint16, 4),
	    ("conversionTime", c_uint16, 1),
	    ("modeOfConversion", c_uint16, 2),
	    ("overflowFlag", c_uint16, 1),
	    ("conversionReady", c_uint16, 1),
	    ("flagHigh", c_uint16, 1),
	    ("flagLow", c_uint16, 1),
	    ("latch", c_uint16, 1),
	    ("polarity", c_uint16, 1),
	    ("maskExponent", c_uint16, 1),
	    ("faultCount", c_uint16, 2)]


class Flags(ctypes.Union):
    _fields_ = [("b", Flag_bits),
		("asshort", c_uint16)]


# ------- config the chip MCP23017 I2C Port Expander --------
# from MCP23017_I2C import *
#	GPIO_CHIP = GPIO_CHIP(0x20, 1)
#	 	    GPIO_CHIP( Device address, Pi Model )
# 0 = Model A, B Rev 2 or B+ Pi ; 1 = Model B Rev 1 Pi)


class MCP230xx:
	def __init__(self, bus_id, device_address):
			self.device_address = device_address  # DEVICE
			self.pi_model = pi_model
			self.bus = smbus.SMBus3(bus_id)

			self.act_ioA = 0x00
			self.act_ioB = 0x00
			self.act_pinA = 0x00
			self.act_pinB = 0x00

			self.IODIRB = 0x01  # Pin direction register
			self.IODIRA = 0x00  # Pin direction register
			self.OLATA = 0x14  # Register for outputs
			self.OLATB = 0x15  # Register for outputs
			self.GPIOA = 0x12  # Register for inputs
			self.GPIOB = 0x13  # Register for inputs

	def setup(self, pin, io, side):
	# GPIO_CHIP.setup( 0, 'OUT', 'A')
	# pin (0,7)  io (IN,OUT)  side (A,B)
		try:

			if (pin < 0 or pin > 7) or (side != 'A' and side != 'B') or (io != 'IN' and io != 'OUT'):
				print ' --- GPIO.setup(pin, io, side) ---'
				print ' --- pin (0 - 7)  io (IN or OUT)  side (A or B) ---'
				return;
			else:

				if io == 'IN' and side == 'A':
					pinio = self.act_ioA | (1 << pin)
					self.act_ioA = pinio
				else:
					if io == 'OUT' and side == 'A':
						pinio = self.act_ioA & ~(1 << pin)
						self.act_ioA = pinio
					else:
						if io == 'IN' and side == 'B':
							pinio = self.act_ioB | (1 << pin)
							self.act_ioB = pinio
						else:
							if io == 'OUT' and side == 'B':
								pinio = self.act_ioB & ~(1 << pin)
								self.act_ioB = pinio

				if side == 'A':
					self.bus.write_byte_data(self.device_address, self.IODIRA, pinio)
				else:
					self.bus.write_byte_data(self.device_address, self.IODIRB, pinio)

			return;

		except:
			print ' --- Error accessing the chip MCP23017 ---'
			raise

	def output(self, pin, hl, side):
	# GPIO_CHIP.output(0, 1, 'A')
	# pin (0,7)  hl (0,1)  side (A,B)
		try:

			if (pin < 0 or pin > 7) or (side != 'A' and side != 'B') or (hl != 1 and hl != 0):
				print ' --- GPIO.output(pin, hl, side) ---'
				print ' --- pin (0 - 7)  hl (0 or 1)  side (A or B) ---'
				return;
			else:
				if hl == 1 and side == 'A':
					pinhl = self.act_pinA | (1 << pin)
					self.act_pinA = pinhl
				else:
					if hl == 0 and side == 'A':
						pinhl = self.act_pinA & ~(1 << pin)
						self.act_pinA = pinhl
					else:
						if hl == 1 and side == 'B':
							pinhl = self.act_pinB | (1 << pin)
							self.act_pinB = pinhl
						else:
							if hl == 0 and side == 'B':
								pinhl = self.act_pinB & ~(1 << pin)
								self.act_pinB = pinhl
				if side == 'A':
					self.bus.write_byte_data(self.device_address, self.OLATA, pinhl)
				else:
					self.bus.write_byte_data(self.device_address, self.OLATB, pinhl)

			return;

		except:
			print ' --- Error accessing the chip MCP23017 ---'
			raise

	def input(self, pin, side):
	# Teste = GPIO_CHIP.input(7, 'B', port_expander)
	# 	pin (0,7)  side (A,B)
		try:
			if (pin < 0 or pin > 7) or (side != 'A' and side != 'B'):
				print ' --- MySwitch = GPIO.input(pin, side) ---'
				print ' --- pin (0 - 7)  side (A or B) ---'
				return;
			else:
				if side == 'A':
					MySwitch = self.bus.read_byte_data(self.device_address, self.GPIOA)
					if MySwitch & (1 << pin) == (1 << pin):
						MySwitch = 1
					else:
						MySwitch = 0
				else:
					MySwitch = self.bus.read_byte_data(self.device_address, self.GPIOB)
					if MySwitch & (1 << pin) == (1 << pin):
						MySwitch = 1
					else:
						MySwitch = 0

			return MySwitch;
		except:
			print ' --- Error accessing the chip MCP23017 ---'
			raise


def main():
	print "=========================================================="
	print "MCP23017 demo for RPI series"
	print "=========================================================="
	time.sleep(0.5)

	bus = 1
	addr = 0x20

	gpio = MCP230xx(bus, addr)


  gpio.setup(0, 'OUT', 'A')
  gpio.setup(1, 'OUT', 'A')
  gpio.setup(2, 'OUT', 'A')
  gpio.setup(3, 'OUT', 'A')
  gpio.setup(4, 'OUT', 'A')
  gpio.setup(5, 'OUT', 'A')
  gpio.setup(6, 'OUT', 'A')
  gpio.setup(7, 'OUT', 'A')
  gpio.setup(0, 'OUT', 'B')
  gpio.setup(1, 'OUT', 'B')
  gpio.setup(2, 'OUT', 'B')
  gpio.setup(3, 'OUT', 'B')
  gpio.setup(4, 'OUT', 'B')
  gpio.setup(5, 'OUT', 'B')
  gpio.setup(6, 'OUT', 'B')
  gpio.setup(7, 'OUT', 'B')



	try:
		while True:
			s="(Ctrl-C to exit)  "
      
      gpio.output(0, 1, 'A')       
      time.sleep(0.5)
      gpio.output(1, 1, 'A')       
      time.sleep(0.5)
      gpio.output(2, 1, 'A')       
      time.sleep(0.5)
      gpio.output(3, 1, 'A')       
      time.sleep(0.5)
      gpio.output(4, 1, 'A')       
      time.sleep(0.5)
      gpio.output(5, 1, 'A')       
      time.sleep(0.5)
      gpio.output(6, 1, 'A')       
      time.sleep(0.5)
      gpio.output(7, 1, 'A')       

      gpio.output(0, 1, 'B')       
      time.sleep(0.5)
      gpio.output(1, 1, 'B')       
      time.sleep(0.5)
      gpio.output(2, 1, 'B')       
      time.sleep(0.5)
      gpio.output(3, 1, 'B')       
      time.sleep(0.5)
      gpio.output(4, 1, 'B')       
      time.sleep(0.5)
      gpio.output(5, 1, 'B')       
      time.sleep(0.5)
      gpio.output(6, 1, 'B')       
      time.sleep(0.5)
      gpio.output(7, 1, 'B')       

      gpio.output(0, 0, 'A')       
      time.sleep(0.5)
      gpio.output(1, 0, 'A')       
      time.sleep(0.5)
      gpio.output(2, 0, 'A')       
      time.sleep(0.5)
      gpio.output(3, 0, 'A')       
      time.sleep(0.5)
      gpio.output(4, 0, 'A')       
      time.sleep(0.5)
      gpio.output(5, 0, 'A')       
      time.sleep(0.5)
      gpio.output(6, 0, 'A')       
      time.sleep(0.5)
      gpio.output(7, 0, 'A') 
      
      gpio.output(0, 0, 'B')       
      time.sleep(0.5)
      gpio.output(1, 0, 'B')       
      time.sleep(0.5)
      gpio.output(2, 0, 'B')       
      time.sleep(0.5)
      gpio.output(3, 0, 'B')       
      time.sleep(0.5)
      gpio.output(4, 0, 'B')       
      time.sleep(0.5)
      gpio.output(5, 0, 'B')       
      time.sleep(0.5)
      gpio.output(6, 0, 'B')       
      time.sleep(0.5)
      gpio.output(7, 0, 'B')      

	except KeyboardInterrupt:
		# Ctrl-C to exit
		sys.exit(0)
		
		
	

if __name__ =="__main__":
    main()      



# **********************************************************************;
# * Project           : RPI_blocks
# *
# * Program name      : SimpleRelay.py
# *
# * Author            : Viacheslav Karpizin,
# *
# * Date created      : 20200330
# *
# * Purpose           : Controlling SimpleRelay with GPIO
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1      original version
# *

#
# Copyright (c) 2019, Viacheslav Karpizin, (c) 2016 Adafruit Industries, Tony DiCola
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# **********************************************************************


import time
import RPi.GPIO as GPIO
import random


GPIO.setmode(GPIO.BOARD)


class SimpleRelay:

    def __init__(self, pin, defaultValue=False, reverse=False):
        self._pin = pin
        self._reverse = reverse

        GPIO.setup(self._pin, GPIO.OUT)
        if defaultValue == True:
            GPIO.output(self._pin, GPIO.HIGH)
        else:
            GPIO.output(self._pin, GPIO.LOW)

    def set(self, value):
        if value == True:
            GPIO.output(self._pin, GPIO.HIGH)
        else:
            GPIO.output(self._pin, GPIO.LOW)


def main():
    print "=========================================================="
    print "SimpleRelay demo for RPI series"
    print "=========================================================="
    time.sleep(2)

    relay = SimpleRelay(40, False, False)

    try:
        while True:
            time.sleep(5)
            relay.set(True)

            time.sleep(5)
            relay.set(False)

    except KeyboardInterrupt:
        # Ctrl-C to exit
        sys.exit(0)


if __name__ == "__main__":
    main()



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : DRV8833.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200314
# *
# * Purpose           : Control both channels of DRV8833 motor driver chip
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1      original version
# *
#
# Copyright (c) 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************
import RPi.GPIO as GPIO
import time
import sys
sys.path.append('./../')


GPIO.setmode(GPIO.BOARD)
# GPIO.setmode(GPIO.BCM)


class DRV8833:
    ain1 = ""
    ain2 = ""
    bin1 = ""
    bin2 = ""

    # Defaults
    freq = 200

    # Constructor

    def __init__(self, ain1, ain2, bin1=none, bin2=none):
        self.ain1 = ain1
        self.ain2 = ain2
        self.bin1 = bin1
        self.bin2 = bin2

        GPIO.setup(ain1, GPIO.OUT)
        GPIO.setup(ain2, GPIO.OUT)
        if bin1 != None:
             GPIO.setup(bin1, GPIO.OUT)
        if bin2 != None:
             GPIO.setup(bin2, GPIO.OUT)

        self.pa = GPIO.PWM(ain2, self.freq)
        self.pb = GPIO.PWM(bin2, self.freq)
        self.pb.start(0)
        self.pa.start(0)


# Speed from -100 to 100

    def drive(self,  speed, motor=1):
        # Negative speed for reverse, positive for forward
        # If necessary use reverse parameter in constructor
        dutyCycle = speed
        if(speed < 0):
            dutyCycle = dutyCycle * -1

        if motor == 1:
            if(speed > 0):
                GPIO.output(self.ain1, GPIO.LOW)
                self.pa.ChangeDutyCycle(dutyCycle)
            else:
                GPIO.output(self.ain1, GPIO.HIGH)
                self.pa.ChangeDutyCycle(100-dutyCycle)
        elif motor == 2 and self.bin1 != None:
            if(speed > 0):
                GPIO.output(self.ain2, GPIO.LOW)
                self.pb.ChangeDutyCycle(dutyCycle)
            else:
                GPIO.output(self.ain2, GPIO.HIGH)
                self.pb.ChangeDutyCycle(100-dutyCycle)
        else:
                print("incorrect motor id")

    def brake(self, motor=1):
        if motor == 1:
            self.pa.ChangeDutyCycle(0)
            GPIO.output(self.ain1, GPIO.LOW)
        else:
            self.pb.ChangeDutyCycle(0)
            GPIO.output(self.bin1, GPIO.LOW)

    def __del__(self):
        GPIO.cleanup()



    def main():
        print("==========================================================")
        print("DRV8833 for RPI Series ")
        print("==========================================================")
        time.sleep(0.5)
	
        # test = DRV8833 (a1, a2, b1, b2, standby, reverse)
        test = DRV8833(15, 16,None, None, None, 11,False)

        try:
            while True:

                print("motor1, forward, 100%")
                test.drive(100) #Forward 100% dutycycle
                time.sleep(3)
					
                print("motor1, forward, 20%")
                test.drive(20) #Forward 100% dutycycle
                time.sleep(3)

                print("motor1, reverse, 100%")
                test.drive(-100) #Backwards 100% dutycycle
                time.sleep(3)

                print("motor1, reverse, 10%")
                test.drive(-10) #Backwards 100% dutycycle
                time.sleep(3)

                print("motor1, brake")
                test.brake() #Short brake
                time.sleep(1)

                print("motor1, standby")
 
                test.standby(True) #Enable standby
                test.standby(False) #Disable standby   
                time.sleep(2)
        except KeyboardInterrupt:
            # Ctrl-C to exit
            sys.exit(0)


if __name__ =="__main__":
    main()   
    



#!/usr/bin/env python


# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : VoltDiv.py
# *
# * Author            : Viacheslav Karpizin,
# *
# * Date created      : 20200317
# *
# * Purpose           : Voltage measurement with ADS1115 + voltage divider
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1      original version
# *

#
# Copyright (c) 2019, Viacheslav Karpizin, D. Scott Williamson
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


# Imports
#
from __future__ import division
import time  # used for sleep function
import sys		# used for print function
import smbus  # I2Cimport spidev
from ads1115 import ADS1115


class VoltDiv:

    ################################################################################
    #
    # Variables
    #

    ################################################################################
    #
    # Functions
    #

    def __init__(self, adcObject, channel, resUp, resDown):

        self._adc = adcObject
        self._channel = channel
        self._resUp = resUp
        self._resDown = resDown

    def readVolt(self):
        value = self._adc.readAdc(self._channel)*4096/3300
        result = value * (self._resUp + self._resDown) / self._resDown
        return result/10000


def main():
    print "=========================================================="
    print "ADS1115 + voltage divider demo for RPI series"
    print "=========================================================="
    time.sleep(1)

    bus = 1
    addr = 0x48

    a = ADS1115(bus, addr)

    v = VoltDiv(a, 1, 30000, 1000)

    print("sensor initialized...")
    try:
        while True:
            s = "(Ctrl-C to exit) ADC Result: "
            val = v.readVolt()
            s += "Voltage: %05.3f" % val+" V  "
            print s
            time.sleep(0.5)
    except KeyboardInterrupt:

        print("interruption")
        # Ctrl-C to exit
        sys.exit(0)


if __name__ == "__main__":
    main()



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : pca9685.py
# *
# * Author            : Viacheslav Karpizin,
# *
# * Date created      : 20200121
# *
# * Purpose           : Manipulate servos and led with PWM
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 201910    Viacheslav Karpizin      1      original version
# *

#
# Copyright (c) 2019, Viacheslav Karpizin, (c) 2016 Adafruit Industries, Tony DiCola
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


from __future__ import division
import logging
import time
import math
import sys
import smbus_m


# Registers/etc:
PCA9685_ADDRESS = 0x40
MODE1 = 0x00
MODE2 = 0x01
SUBADR1 = 0x02
SUBADR2 = 0x03
SUBADR3 = 0x04
PRESCALE = 0xFE
LED0_ON_L = 0x06
LED0_ON_H = 0x07
LED0_OFF_L = 0x08
LED0_OFF_H = 0x09
ALL_LED_ON_L = 0xFA
ALL_LED_ON_H = 0xFB
ALL_LED_OFF_L = 0xFC
ALL_LED_OFF_H = 0xFD

# Bits:
RESTART = 0x80
SLEEP = 0x10
ALLCALL = 0x01
INVRT = 0x10
OUTDRV = 0x04


logger = logging.getLogger(__name__)


# def software_reset(i2c=None, **kwargs): #TODO fix for specifix addresses
#    """Sends a software reset (SWRST) command to all servo drivers on the bus."""
#    # Setup I2C interface for device 0x00 to talk to all of them.
#    if i2c is None:
#        import Adafruit_GPIO.I2C as I2C
#        i2c = I2C
#    self._device = i2c.get_i2c_device(0x00, **kwargs)
#    self._device.writeRaw8(0x06)  # SWRST


class PCA9685(object):
    """PCA9685 PWM LED/servo controller."""

    def __init__(self, address=PCA9685_ADDRESS, bus_id=0, **kwargs):
        """Initialize the PCA9685."""
        # Setup I2C interface for the device.
        self.bus_id = bus_id
        self.bus = smbus_m.SMBus3(bus_id)
        self.addr = address

        self.set_all_pwm(0, 0)
        self.bus.write_byte_data(self.addr, MODE2, OUTDRV)
        self.bus.write_byte_data(self.addr, MODE1, ALLCALL)
        time.sleep(0.005)  # wait for oscillator
        mode1 = self.bus.read_byte_data(self.addr, MODE1)
        print(mode1)
        mode1 = mode1 & ~SLEEP  # wake up (reset sleep)
        print(mode1)
        self.bus.write_byte_data(self.addr, MODE1, mode1)
        time.sleep(0.005)  # wait for oscillator

    def set_pwm_freq(self, freq_hz):
        """Set the PWM frequency to the provided value in hertz."""
        prescaleval = 25000000.0    # 25MHz
        prescaleval /= 4096.0       # 12-bit
        prescaleval /= float(freq_hz)
        prescaleval -= 1.0
        logger.debug('Setting PWM frequency to {0} Hz'.format(freq_hz))
        logger.debug('Estimated pre-scale: {0}'.format(prescaleval))
        prescale = int(math.floor(prescaleval + 0.5))
        logger.debug('Final pre-scale: {0}'.format(prescale))
        oldmode = self.bus.read_byte_data(self.addr, MODE1)
        print(oldmode)
        newmode = (oldmode & 0x7F) | 0x10    # sleep
        print(newmode)
        self.bus.write_byte_data(self.addr, MODE1, newmode)  # go to sleep
        self.bus.write_byte_data(self.addr, PRESCALE, prescale)
        self.bus.write_byte_data(self.addr, MODE1, oldmode)
        print(prescale)
        time.sleep(0.005)
        self.bus.write_byte_data(self.addr, MODE1, oldmode | 0x80)

    def set_pwm(self, channel, on, off):
        """Sets a single PWM channel."""
        self.bus.write_byte_data(self.addr, LED0_ON_L+4*channel, on & 0xFF)
        self.bus.write_byte_data(self.addr, LED0_ON_H+4*channel, on >> 8)
        self.bus.write_byte_data(self.addr, LED0_OFF_L+4*channel, off & 0xFF)
        self.bus.write_byte_data(self.addr, LED0_OFF_H+4*channel, off >> 8)

    def set_all_pwm(self, on, off):
        """Sets all PWM channels."""
        self.bus.write_byte_data(self.addr, ALL_LED_ON_L, on & 0xFF)
        self.bus.write_byte_data(self.addr, ALL_LED_ON_H, on >> 8)
        self.bus.write_byte_data(self.addr, ALL_LED_OFF_L, off & 0xFF)
        self.bus.write_byte_data(self.addr, ALL_LED_OFF_H, off >> 8)


def main():
    print "=========================================================="
    print "PCA9685 demo for RPI series"
    print "=========================================================="
    time.sleep(3)

    # logging.basicConfig(level=logging.DEBUG)

    # Initialise the PCA9685 using the default address (0x40).
    #pwm = Adafruit_PCA9685.PCA9685()

    # Alternatively specify a different address and/or bus:
    pwm = PCA9685(address=0x40, busnum=0)

    # Configure min and max servo pulse lengths
    servo_min = 150  # Min pulse length out of 4096
    servo_max = 600  # Max pulse length out of 4096

    # Set frequency to 60hz, good for servos.
    pwm.set_pwm_freq(60)

    print('Moving servo on channel 0, press Ctrl-C to quit...')
    while True:
        # Move servo on channel O between extremes.
        pwm.set_all_pwm(0, servo_min)
        time.sleep(1)
        pwm.set_all_pwm(0, servo_max)
        time.sleep(1)


if __name__ == "__main__":
    main()



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : L9110H.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200330
# *
# * Purpose           : Control motor via L9110 driver
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1      original version
# *
#
# Copyright (c) 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************
import RPi.GPIO as GPIO
import time
import sys
sys.path.append('./../')


GPIO.setmode(GPIO.BOARD)
# GPIO.setmode(GPIO.BCM)


class DRV8833:
	ain1 = ""
	ain2 = ""

	# Defaults
	freq = 200

	# Constructor

	def __init__(self, ain1, ain2):
		self.ain1 = ain1
		self.ain2 = ain2

		GPIO.setup(ain1, GPIO.OUT)
		GPIO.setup(ain2, GPIO.OUT)


    self.pa = GPIO.PWM(ain2, self.freq)

		self.pa.start(0)


# Speed from -100 to 100
	def drive(self,  speed ):
		# Negative speed for reverse, positive for forward
		# If necessary use reverse parameter in constructor
		dutyCycle = speed
		if(speed < 0):
			dutyCycle = dutyCycle * -1



      			if(speed > 0):
        			GPIO.output(self.ain1,GPIO.LOW)
        			self.pa.ChangeDutyCycle(dutyCycle)
      			else:
        			GPIO.output(self.ain1,GPIO.HIGH)
        			self.pa.ChangeDutyCycle(100-dutyCycle)



	def brake(self):

			self.pa.ChangeDutyCycle(0)
			GPIO.output(self.ain1,GPIO.LOW)



	def __del__(self):
		GPIO.cleanup()
    
    
    
 def main():
	print "=========================================================="
	print "L9110H demo for RPI Series "
	print "=========================================================="
	time.sleep(3)
	

  	test = L9110(15, 16)

	
	

	try:
		while True:

			      print(" forward, 100%")
      			test.drive(100) #Forward 100% dutycycle
      			time.sleep(3)


					
            print(" forward, 20%")
            test.drive(20) #Forward 100% dutycycle
            time.sleep(3)

      			print(" reverse, 100%")
      			test.drive(-100) #Backwards 100% dutycycle
      			time.sleep(3)

            print(" reverse, 10%")
            test.drive(-10) #Backwards 100% dutycycle
            time.sleep(3)


      
     	 		  print(" brake")
      			test.brake() #Short brake
      			time.sleep(1)


	except KeyboardInterrupt:
		# Ctrl-C to exit
		sys.exit(0)


if __name__ =="__main__":
    main()   



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : mcp410xx.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200306
# *
# * Purpose           : Read battery parameters from 3.7 lithium battery (used in UPS Lite module for RPI Zero)
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1      original version
# *
#
# Copyright (c) 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************

# load the spidev library
import spidev
import struct

# initialize spi
spi = spidev.SpiDev()


class MCP410xxPot:

         def __init__(self, bus=0, device=0):


                    # open the spi device
                spi.open(bus, device)
  spi.max_speed_hz = 500000

         def set_wiper(self, value = 0.5):
                """ Sets the wiper position

            input:
            -----

                value   : a value between 0 and 1 that sets the 
                          relative position of the potentiometer

            output:
            ------

                sends an SPI signal to the digipot to set the relative
                position of the wiper

        """

                # set the bounds of the value
                value = max([0, value])
                value = min([value, 1])

                # scale the value
                scaled_value = int(value * 255)

              # set the wiper write bit
                wiper_write = int(0b00010001)

                # send the wiper bit
                spi.xfer([wiper_write, scaled_value])


 
        def set_shutdown(self):
                """ Sets the total resistance

            input:
            -----

                value   : a value between 0 and 1 that sets the 
                          relative position of the potentiometer

            output:
            ------

                sends an SPI signal to the digipot to set the resistor network

        """

                # set the wiper write bit
                wiper_write = int(0b00100001)

                # send the wiper bit
                spi.xfer([wiper_write, 0])


if __name__ == "__main__":
            # get the wiper setting from the command line
        #import sys
        #val = float(sys.argv[1])

  print "=========================================================="
   print "MCP410xx demo for RPI series"
    print "Changing value from 0 to max value every 0.5 sec with 1/100 step"
    print "=========================================================="
    time.sleep(0.5)
        # test the wiper setting function
        mypot = MCP410xxPot()
        mypot.set_wiper(val)

  foreach i in (0..100):
       mypot.set_wiper(i/100)
        time.sleep(0.5)



import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from .ads1115 import *
from xpi_blocks.smbus_m import *
#from DS1302 import *
#from ds1307 import *



# The MIT License (MIT)
#
# Copyright (c) 2015 Nick Lee
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from collections import namedtuple

class Temperature(namedtuple('Temperature', ['C', 'F', 'K'])):
    def __new__(cls, **kwargs):
        if len(kwargs) != 1:
            raise RuntimeError('Specify temperature in one and only one unit: C, F, K')

        if 'C' in kwargs:
            kwargs['F'] = kwargs['C'] * 1.8 + 32
            kwargs['K'] = kwargs['C'] + 273.15
        elif 'F' in kwargs:
            kwargs['C'] = (kwargs['F'] - 32) / 1.8
            kwargs['K'] = kwargs['C'] + 273.15
        elif 'K' in kwargs:
            kwargs['C'] = kwargs['K'] - 273.15
            kwargs['F'] = kwargs['C'] * 1.8 + 32
        else:
            raise RuntimeError('Specify temperature in either: C, F, K')

        return super(Temperature, cls).__new__(cls, **kwargs)

class Humidity(namedtuple('Humidity', ['RH'])):
    pass

class Altitude(namedtuple('Altitude', ['m', 'ft'])):
    def __new__(cls, **kwargs):
        if len(kwargs) != 1:
            raise RuntimeError('Specify altitude in one and only one unit: m, ft')

        if 'm' in kwargs:
            kwargs['ft'] = kwargs['m'] * 3.28083998
        elif 'ft' in kwargs:
            kwargs['m'] = kwargs['ft'] * 0.3048
        else:
            raise RuntimeError('Specify altitude in either: m, ft')

        return super(Altitude, cls).__new__(cls, **kwargs)

class Pressure(namedtuple('Pressure', ['hPa'])):
    def altitude(self, msl):
        msl = msl.hPa if type(msl) is Pressure else msl
        m = 44330 * (1 - (self.hPa / msl)**(1 / 5.255))
        return Altitude(m=m)

    def msl_pressure(self, altitude):
        m = altitude.m if type(altitude) is Altitude else altitude
        hPa = self.hPa / (1.0 - m/44330.0)**5.255
        return Pressure(hPa=hPa)



#!/usr/bin/env python

#################################################
# Code for MAX44009 (GY-49) light sensor
#
#
#
#
#################################################

import smbus


class MAX44009():

    _REG_INTERRUPT_STATUS = 0x00
    _REG_INTERRUPT_ENABLE = 0x01
    _REG_CONFIGURATION = 0x02
    _REG_LUX_HIGH_BYTE = 0x03
    _REG_LUX_LOW_BYTE = 0x04
    _REG_UPPER_THRESHOLD = 0x05
    _REG_LOWER_THRESHOLD = 0x06
    _REG_TIMER_THRESHOLD = 0x07

    def __init__(self, bus=1, addr=0x4a):
        self._bus = smbus.SMBus(bus)
        self._addr = addr

    def _write(self, register, data):
        self._bus.write_byte_data(self._addr, register, data)

    def _read(self, register):
        return self._bus.read_byte_data(self._addr, register)

    def _read_block(self, register, size=2):
        return self._bus.read_i2c_block_data(self._addr, register, size)

    def configure(self, cont=0, manual=0, cdr=0, timer=0):
        configuration = (cont & 0x01) << 7 | (
            manual & 0x01) << 6 | (cdr & 0x01) << 3 | timer & 0x07
        self._write(self._REG_CONFIGURATION, configuration)

    def luminosity(self):
        data = self._read_block(self._REG_LUX_HIGH_BYTE, 2)
        exponent = (data[0] & 0xF0) >> 4
        mantissa = ((data[0] & 0x0F) << 4) | (data[1] & 0x0F)
        luminance = ((2 ** exponent) * mantissa) * 0.045
        return luminance



# DRAFT


# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : ShiftRegister74HC165.py
# *
# * Author            : Viacheslav Karpizin,
# *
# * Date created      : 20200407
# *
# * Purpose           : Input Shift Register 74HC165 (parallel to serial)
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202004    Viacheslav Karpizin      1      original version
# *

#
# Copyright (c) 2020, Viacheslav Karpizin,
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


import RPi.GPIO as io
import time


class ShiftReg():

    '''
    This class contains the code that on a low level handles interfacing with the register.
    It takes care of loading values into the register and reading the values out of the register
    '''

    def __init__(self, serial_out, load_pin, clock_enable, clock_pin, warnings=False, bitcount=8):
        '''
        :param serial_out: BCM GPIO pin that's connected to chip pin 9 (Serial Out)
        :param load_pin: BCM GPIO pin that's connected to chip pin 1 (PL, Shift/Load)
        :param clock_enable: BCM GPIO pin that's connected to chip pin 15 (Clock enable, Chip enable) not required
        :param clock_pin: BCM GPIO pin that's connected to chip pin 2 (Clock Pin
        :param warnings:
        :param bitcount: 8 for 1 chip, 16 for 2 chip one after another and so on
        '''

        io.setwarnings(warnings)
        io.setmode(io.BCM)
        self.data_pin = serial_out
        self.load_reg_pin = load_pin

        self.clock_pin = clock_pin
        self.clock_enable = clock_enable
        self._gpio_init()

        self.bitcount = bitcount

    def __enter__(self):

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):

        self.clean_gpio()

    def _gpio_init(self):
        '''Sets up GPIO pins for output and gives them their default value
        :return: Nothing
        '''

        # sets GPIO pins for output
        io.setup(self.data_pin, io.IN, pull_up_down=io.PULL_DOWN)
        io.setup(self.clock_pin, io.OUT)
        io.setup(self.clock_enable, io.OUT)
        io.setup(self.load_reg_pin, io.OUT)

        # sets default values on GPIO pins

        io.output(self.clock_pin, 0)
        io.output(self.clock_enable, 0)
        io.output(self.load_reg_pin, 1)

    def _read_input(self):
        '''This method will read the value on the serial out pin from the
        register
        :return: Value read from the register
        '''

        status = io.input(self.data_pin)

        return status

    def _cycle_clock(self, n=1):
        '''This method will cycle the clock pin high and low to shift
        the data down the register
        :param n: Number of times to cycle clock (Default 1 time)
        :return:
        '''

        self._shift_register(n)

    def _shift_register(self, n=1):
        ''' This method cycles the clock pin high and low which
        shifts the register down.
        :param n: number of times to cycle the clock pin (default 1 time)
        :return:
        '''

        for x in range(n):
            io.output(self.clock_pin, 1)
            io.output(self.clock_pin, 0)

        def read_input(self, bit):
            register = read_register()
            if bit < 0 or bit >= self.bitcount:
                raise IndexError
            value = register[bit]
            return value

    def read_register(self):
        '''This method handles reading the data out of the entire register.
        It loads the values into the register, shifts all of the values out of the
        register and reads the values storing them in a list left to right starting
        at Pin 0 going to the highest pin in your chain
        :return: List of pin values from the register lowest to highest pin.
        '''

        register = []

        # Loads the status of the input pins into the internal register
        self._load_register()

        # shifts out each bit and stores the value
        for x in range(self.bitcount):
            # Stores the value of the pin
            register.append(self._read_input())

            # Cycles the clock causing the register to shift
            self._shift_register()

        # reverses the list so it reads from left to right pin0 - pin7
        register.reverse()

        return register

    def _load_register(self):
        '''This method takes the values on the input pins of the register and loads
        them into the internal storage register in preperation to be shifted out of the
        serial port
        :return: Nothing
        '''

        io.output(self.load_reg_pin, 0)
        io.output(self.load_reg_pin, 1)

    def clean_gpio(self):
        '''This method cleans up the GPIO pins, it should be run whenever you are done
        interfacing with the GPIO pins like at the end of the script/program.
        :return:
        '''

        io.cleanup()


if __name__ == '__main__':

    '''
    This class serves as an example how to interface with the code above when trying to drop into a
    loop to read the input from a register. All we need to do is overwrite the handle_on_up and 
    handle_on_down methods and we can read input from our input register and dont have to worry
    about any low level handling.
    '''
    class Test(ReadHandler):
        def __init__(self, serial_out, load_pin, clock_enable, clock_pin, warnings=False, bitcount=8):
            ReadHandler.__init__(self, serial_out, load_pin,
                                 clock_enable, clock_pin, warnings, bitcount)

        def handle_on_up(self, pin):
            print('Pin {} has been changed to UP'.format(pin))

        def handle_on_down(self, pin):
            print('Pin {} has been changed to DOWN'.format(pin))

    with Test(26, 4, 6, 5) as t:
        try:
            t.watch_inputs()
        except KeyboardInterrupt:
            t.loop_breaker = True
            print('\nBroke Loop')
    '''
    t = ShiftReg(26, 4, 6, 5)
    while True:
        try:
            reg = t.read_register()
            print(reg, end='\r')
            time.sleep(.2)
        except KeyboardInterrupt:
            break
    t.clean_gpio()
    '''


def main():
    print "=========================================================="
    print "74HC165 shift register (parallel in, serial out) demo for RPI series"
    print "=========================================================="
    time.sleep(1)

    c = HC165(26, 4, 6, 5)

    print("chip initialized...")
    try:
        while True:

            reg = t.read_register()
            print(reg, end='\r')
            time.sleep(2)
    except KeyboardInterrupt:
        break

    t.clean_gpio()


if __name__ == "__main__":
    main()




# **********************************************************************;
# * Project           : RPI_Blocks
# *
# * Program name      : HDC1080.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200410
# *
# * Purpose           : HDC1080 (CCS811) gas sensor
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202004    Viacheslav Karpizin      1      original version
# *
#
# Copyright (c) 2019, Viacheslav Karpizin, 2017 (c) Sai Yamanoor
#

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


import time


class CCS811(object):

    def __init__(self, device_bus=1, device_address=0x5A):
        self.device_bus = device_bus
        self.device_address = device_address
        self.bus = I2C("/dev/i2c-1")

    def start_app(self):
        self.write_byte(0xF4)
        print(self.read_byte(0x00))
        self.write_byte_data(0x01, 0x10)

    def transfer(self, msgs):
        try:
            self.bus.transfer(self.device_address, msgs)
        except I2CError as error:
            print(str(error))
        return msgs

    def close(self):
        self.bus.close()

    def read_byte(self, address):
        msgs = [I2C.Message([address], read=False)]
        self.transfer(msgs)
        msgs = [I2C.Message([address], read=True)]
        ret_msg = self.transfer(msgs)

        return (ret_msg[0].data[0])

    def write_byte(self, address):
        msgs = [I2C.Message([address], read=False)]
        self.transfer(msgs)

    def reset(self):
        msgs = [I2C.Message([0xFF, 0x11, 0xE5, 0x72, 0x8A], read=False)]
        self.transfer(msgs)

    def write_byte_data(self, address, data):
        msgs = [I2C.Message([address, data], read=False)]
        self.transfer(msgs)

    def read_bytes(self, count):
        msgs = [I2C.Message([0] * count, read=True)]
        ret_msg = self.transfer(msgs)
        return ret_msg[0].data


if __name__ == "__main__":
    my_ccs811 = CCS811()
    my_ccs811.reset()

    print(my_ccs811.read_byte(0x00))
    print(my_ccs811.read_byte(0x20))

    my_ccs811.start_app()

    my_ccs811.read_byte(0x00)
    measurement_time = time.time()

    while True:
        if my_ccs811.read_byte(0x00) == 152:
            my_ccs811.write_byte(0x02)
            data = my_ccs811.read_bytes(8)

            eco2 = data[0] << 8 | data[1]
            voc = data[2] << 8 | data[3]
            print(eco2, voc)

            if (time.time() - measurement_time) > 900:
                measurement_time = time.time()
                post_data([eco2, voc])
        else:
            if my_ccs811.read_byte(0x00) & 0x01:
                my_ccs811.close()
                my_ccs811 = CCS811()
                my_ccs811.reset()
                time.sleep(1)
                my_ccs811.start_app()




# **********************************************************************;
# * Project           : xPILib
# *
# * Program name      : at24cXXX.py (AT24C64, AT24C128, AT24C256)
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20191121
# *
# * Purpose           : Managing I2C EEPROM memory module
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 201911    Viacheslav Karpizin      1      original version
# *
#
# Copyright (c) 2019, Viacheslav Karpizin
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************


# AT24C32A, 32K (32768 kbit / 4 KB), 128 pages, 32 bytes per page, i2c addr 0x50

import time


class AT24C32N(object):
    """Driver for the AT24C32N 32K EEPROM."""

    def __init__(self, i2c, i2c_addr=0x50, pages=128, bpp=32):
        self.i2c = i2c
        self.i2c_addr = i2c_addr
        self.pages = pages
        self.bpp = bpp  # bytes per page

    def capacity(self):
        """Storage capacity in bytes"""
        return self.pages * self.bpp

    def read(self, addr, nbytes):
        """Read one or more bytes from the EEPROM starting from a specific address"""
        return self.i2c.readfrom_mem(self.i2c_addr, addr, nbytes, addrsize=16)

    def write(self, addr, buf):
        """Write one or more bytes to the EEPROM starting from a specific address"""
        offset = addr % self.bpp
        partial = 0
        # partial page write
        if offset > 0:
            partial = self.bpp - offset
            self.i2c.writeto_mem(self.i2c_addr, addr,
                                 buf[0:partial], addrsize=16)
            time.sleep_ms(5)
            addr += partial
        # full page write
        for i in range(partial, len(buf), self.bpp):
            self.i2c.writeto_mem(self.i2c_addr, addr+i -
                                 partial, buf[i:i+self.bpp], addrsize=16)
            time.sleep_ms(5)


# define FULL_MASK 0x7FFFF
# define DEVICE_MASK 0x7F0000
# define WORD_MASK 0xFFFF

class AT24CXXX:
  def __init__(self, bus_id, addr):
		self._address = addr
    self._bus = smbus_m.SMBus3(bus_id)
    return
    
    
    
  def write(self, dataAddress, data):
    Wire.beginTransmission((uint8_t)((0x500000 | dataAddress) >> 16)); // B1010xxx
    self._bus.write_byte(self._address, (0x500000 | dataAddress) >> 16)
    Wire.write((uint8_t)((dataAddress & WORD_MASK) >> 8)); // MSB
    Wire.write((uint8_t)(dataAddress & 0xFF)); // LSB
    Wire.write(data);
    Wire.endTransmission();
    delay(5);    
    
    
  def read(self, dataAddress):
    uint8_t data = 0x00;
    Wire.beginTransmission((uint8_t)((0x500000 | dataAddress) >> 16)); // B1010xxx
    Wire.write((uint8_t)((dataAddress & WORD_MASK) >> 8)); // MSB
    Wire.write((uint8_t)(dataAddress & 0xFF)); // LSB
    Wire.endTransmission();
    Wire.requestFrom((uint8_t)((0x500000 | dataAddress) >> 16),(uint8_t)1);
    if (Wire.available()) 
      data = Wire.peek()
    return data;    
    


def main():
	print("==========================================================")
	print("AT24CXXX demo for RPI series")
	print("==========================================================")
	time.sleep(0.5)
	
	bus = 1
	addr =  0x44
	
	memChip = AT24CXXX(bus, addr)
	


	try:
		while True:

	except KeyboardInterrupt:
		# Ctrl-C to exit
		sys.exit(0)
		
		
	

if __name__ =="__main__":
    main()






# **********************************************************************;
# * Project           : RPI_blocks
# *
# * Program name      : ULN2003StepperMotorUnipolar.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200403
# *
# * Purpose           : Control Unipolar stepper motor with uln2003 darlington driver
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202004    Viacheslav Karpizin      1       original version
# *
#
# Copyright (c) 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# http://robocraft.ru/files/datasheet/28BYJ-48.pdf
# **********************************************************************
import RPi.GPIO as GPIO
import time
import smbus_m
import sys
sys.path.append('./../')


# GPIO.setmode(GPIO.BOARD)
GPIO.setmode(GPIO.BCM)


class ULN2003StepperUniPolar:

    CW = True
    CCW = False

    # int Steps = 0
    # boolean Direction = true# gre
    # unsigned long last_time
    # unsigned long currentMillis
    stepsLeft = 4095
    # long time

    _steps = 0

    def rotateHalfStep(self, steps, direction):
        self._setDirection(direction)
        for x in range(0, steps):

            if self._steps == 0:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.HIGH)
            elif self._steps == 1:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.HIGH)
                GPIO.output(self._pin4, GPIO.HIGH)
            elif self._steps == 2:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.HIGH)
                GPIO.output(self._pin4, GPIO.LOW)
            elif self._steps == 3:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.HIGH)
                GPIO.output(self._pin3, GPIO.HIGH)
                GPIO.output(self._pin4, GPIO.LOW)
            elif self._steps == 4:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.HIGH)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)
            elif self._steps == 5:
                GPIO.output(self._pin1, GPIO.HIGH)
                GPIO.output(self._pin2, GPIO.HIGH)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)
            elif self._steps == 6:
                GPIO.output(self._pin1, GPIO.HIGH)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)
            elif self._steps == 7:
                GPIO.output(self._pin1, GPIO.HIGH)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.HIGH)
            else:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)

            self.adjustSteps()

    def rotateFullStep(self, steps, direction):
        self._setDirection(direction)
        for x in range(0, steps):

            if self._steps == 0:
                GPIO.output(self._pin1, GPIO.HIGH)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.HIGH)
            elif self._steps == 1:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.HIGH)
                GPIO.output(self._pin4, GPIO.HIGH)
            elif self._steps == 2:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.HIGH)
                GPIO.output(self._pin3, GPIO.HIGH)
                GPIO.output(self._pin4, GPIO.LOW)
            elif self._steps == 3:
                GPIO.output(self._pin1, GPIO.HIGH)
                GPIO.output(self._pin2, GPIO.HIGH)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)
            else:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)

            self.adjustSteps(4)

    def rotateOnePhase(self, steps, direction):
        self._setDirection(direction)
        for x in range(0, steps):

            if self._steps == 0:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.HIGH)
            elif self._steps == 1:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.HIGH)
                GPIO.output(self._pin4, GPIO.LOW)
            elif self._steps == 2:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.HIGH)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)
            elif self._steps == 3:
                GPIO.output(self._pin1, GPIO.HIGH)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)
            else:
                GPIO.output(self._pin1, GPIO.LOW)
                GPIO.output(self._pin2, GPIO.LOW)
                GPIO.output(self._pin3, GPIO.LOW)
                GPIO.output(self._pin4, GPIO.LOW)

            self.adjustSteps(4)

    def adjustSteps(self, totalSteps=8):
        if self._direction == True:
            self._steps = self._steps+1
        if self._direction == False:
            self._steps = self._steps-1
        if self._steps > totalSteps-1:
            self._steps = 0
        if self._steps < 0:
            self._steps = totalSteps-1

    def _setDirection(self, direction):
        self._direction = direction

    # Constructor
    def __init__(self, pin1, pin2, pin3, pin4):
        self._pin1 = pin1
        self._pin2 = pin2
        self._pin3 = pin3
        self._pin4 = pin4
        GPIO.setup(self._pin1, GPIO.OUT)
        GPIO.setup(self._pin2, GPIO.OUT)
        GPIO.setup(self._pin3, GPIO.OUT)
        GPIO.setup(self._pin4, GPIO.OUT)

        GPIO.output(self._pin1, GPIO.LOW)
        GPIO.output(self._pin2, GPIO.LOW)
        GPIO.output(self._pin3, GPIO.LOW)
        GPIO.output(self._pin4, GPIO.LOW)

    # def rotate(self, direction, steps ):

    def __del__(self):
        GPIO.cleanup()


def main():
    print "=========================================================="
    print "Unipolar Stepper Motor Handling with ULN2003 for RPI Series "
    print "=========================================================="
    time.sleep(0.5)

    stepsLeft = 4095

    stepper = ULN2003StepperUniPolar(5, 6, 13, 19)

    direction = ULN2003StepperUniPolar.CW

    try:
        while True:
            print("==================================================== ")
            print("Half stepping ")
            while stepsLeft > 0:
                time.sleep(0.01)

                stepper.rotateHalfStep(1, direction)
                stepsLeft = stepsLeft-1

            print("Change direction... ")
            time.sleep(2)
            direction = not direction
            stepsLeft = 4095

            while stepsLeft > 0:
                time.sleep(0.01)

                stepper.rotateHalfStep(1, direction)
                stepsLeft = stepsLeft-1

            print("Change direction... ")
            time.sleep(2)
            direction = not direction
            stepsLeft = 4095

            print("==================================================== ")
            print("full stepping ")
            while stepsLeft > 0:
                time.sleep(0.01)

                stepper.rotateFullStep(1, direction)
                stepsLeft = stepsLeft-1

            print("Change direction... ")
            time.sleep(2)
            direction = not direction
            stepsLeft = 4095

            while stepsLeft > 0:
                time.sleep(0.01)

                stepper.rotateFullStep(1, direction)
                stepsLeft = stepsLeft-1

            print("Change direction... ")
            time.sleep(2)
            direction = not direction
            stepsLeft = 4095

            print("==================================================== ")
            print("one phase  ")
            while stepsLeft > 0:
                time.sleep(0.01)

                stepper.rotateOnePhase(1, direction)
                stepsLeft = stepsLeft-1

            print("Change direction... ")
            time.sleep(2)
            direction = not direction
            stepsLeft = 4095

            while stepsLeft > 0:
                time.sleep(0.01)

                stepper.rotateOnePhase(1, direction)
                stepsLeft = stepsLeft-1

            print("Change direction... ")
            time.sleep(2)
            direction = not direction
            stepsLeft = 4095
    except KeyboardInterrupt:
        # Ctrl-C to exit
        sys.exit(0)


if __name__ == "__main__":
    main()



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : TB6612FNG.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200313
# *
# * Purpose           : Control both channels of TB6612 motor driver chip
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1       original version
# *
#
# Copyright (c) 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.sparkfun.com/datasheets/Robotics/TB6612FNG.pdf
# **********************************************************************
import RPi.GPIO as GPIO
import time
import smbus_m
import sys
sys.path.append('./../')


GPIO.setmode(GPIO.BOARD)
# GPIO.setmode(GPIO.BCM)


class Motor:
    ain1 = ""
    ain2 = ""
    apwm = ""
    bin1 = ""
    bin2 = ""
    bpwm = ""
    standbyPin = ""

    # Defaults
    freq = 200
    reverse = False  # Reverse flips the direction of the motor

    # Constructor
    def __init__(self, ain1, ain2, apwm, bin1=None, bin2=None, bpwm=None, standbyPin=None, reverse=False):
        self.ain1 = ain1
        self.ain2 = ain2
        self.apwm = apwm
        self.bin1 = bin1
        self.bin2 = bin2
        self.bpwm = bpwm

        self.standbyPin = standbyPin
        self.reverse = reverse

        GPIO.setup(ain1, GPIO.OUT)
        GPIO.setup(ain2, GPIO.OUT)
        GPIO.setup(apwm, GPIO.OUT)
        if (bin1 != None):
            GPIO.setup(bin1, GPIO.OUT)
        if bin2 != None:
            GPIO.setup(bin2, GPIO.OUT)
        if bpwm != None:
            GPIO.setup(bpwm, GPIO.OUT)
            self.pb = GPIO.PWM(bpwm, self.freq)
            self.pb.start(0)

        if standbyPin != None:
            GPIO.setup(standbyPin, GPIO.OUT)
            GPIO.output(standbyPin, GPIO.HIGH)

        self.pa = GPIO.PWM(apwm, self.freq)
        self.pa.start(0)

    # Speed from -100 to 100
    def drive(self,  speed, motor=1):
        # Negative speed for reverse, positive for forward
        # If necessary use reverse parameter in constructor
        dutyCycle = speed
        if(speed < 0):
            dutyCycle = dutyCycle * -1

        if(self.reverse):
            speed = speed * -1
        if motor == 1:
            if(speed > 0):
                print("speed + for motor1")
                GPIO.output(self.ain1, GPIO.HIGH)
                GPIO.output(self.ain2, GPIO.LOW)
            else:
                print("speed - for motor1")
                GPIO.output(self.ain1, GPIO.LOW)
                GPIO.output(self.ain2, GPIO.HIGH)
            self.pa.ChangeDutyCycle(dutyCycle)
        elif motor == 2 and self.bin1 != None:
            if(speed > 0):
                GPIO.output(self.bin1, GPIO.HIGH)
                GPIO.output(self.bin2, GPIO.LOW)
            else:
                GPIO.output(self.bin1, GPIO.LOW)
                GPIO.output(self.bin2, GPIO.HIGH)
            self.pb.ChangeDutyCycle(dutyCycle)
        else:
            print("incorrect motor id")

    def brake(self, motor=1):
        if motor == 1:
            self.pa.ChangeDutyCycle(0)
            GPIO.output(self.ain1, GPIO.HIGH)
            GPIO.output(self.ain2, GPIO.HIGH)
        else:
            self.pb.ChangeDutyCycle(0)
            GPIO.output(self.bin1, GPIO.HIGH)
            GPIO.output(self.bin2, GPIO.HIGH)

    def standby(self, value):
        self.pa.ChangeDutyCycle(0)
        if self.bin1 != None:
            self.pb.ChangeDutyCycle(0)
        GPIO.output(self.standbyPin, value)

    def __del__(self):
        GPIO.cleanup()


def main():
    print "=========================================================="
    print "TB6612FNG for RPI Series "
    print "=========================================================="
    time.sleep(0.5)

    #test = Motor (a1, a2, apwm, b1, b2, bpwm, standby, reverse)
    test = Motor(15, 16, 13, None, None, None, 11, False)
    # blue - GND
    # violet, grey - motor1
    # violet, white - 5V(VM), 3v3 (VCC)
    # orange - pwm a (13, GPIO27)
    # yellow - a2 (16 GPIO23)
    # green  - a1 (15 GPIO22)
    # black   - standby (11, GPIO17)

    try:
        while True:

            print("motor1, forward, 100%")
            test.drive(100)  # Forward 100% dutycycle
            time.sleep(3)

            print("motor1, forward, 20%")
            test.drive(20)  # Forward 100% dutycycle
            time.sleep(3)

            print("motor1, reverse, 100%")
            test.drive(-100)  # Backwards 100% dutycycle
            time.sleep(3)

            print("motor1, reverse, 10%")
            test.drive(-10)  # Backwards 100% dutycycle
            time.sleep(3)

            print("motor1, brake")
            test.brake()  # Short brake
            time.sleep(1)

            print("motor1, standby")

            test.standby(True)  # Enable standby
            test.standby(False)  # Disable standby
            time.sleep(2)
    except KeyboardInterrupt:
        # Ctrl-C to exit
        sys.exit(0)


if __name__ == "__main__":
    main()



import setuptools
from setuptools import find_packages


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="xpi_blocks", 
    version="0.0.18",
    author="Viacheslav Karpizin",
    author_email="karpizin@gmail.com",
    description="Package for xPI boards to handle sensors and peripherial devices",
    #url="https://github.com/xtile/rpi_blocks",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
    zip_safe=False
)



# **********************************************************************;
# * Project           : xPI.blocks
# *
# * Program name      : ds1307.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200330
# *
# * Purpose           : Handling RTC clock ds1307 and DS3231N with Python
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202003    Viacheslav Karpizin      1      original version
# *
#
# Copyright (c) 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# **********************************************************************

import smbus_m
from datetime import datetime
import time
import sys
sys.path.append('./../')


def _bcd_to_int(bcd):
    """Decode a 2x4bit BCD to a integer.
    """
    out = 0
    for d in (bcd >> 4, bcd):
        for p in (1, 2, 4, 8):
            if d & 1:
                out += p
            d >>= 1
        out *= 10
    return out / 10


def _int_to_bcd(n):
    """Encode a one or two digits number to the BCD.
    """
    bcd = 0
    for i in (n // 10, n % 10):
        for p in (8, 4, 2, 1):
            if i >= p:
                bcd += 1
                i -= p
            bcd <<= 1
    return bcd >> 1


class DS1307:

    _REG_SECONDS = 0x00
    _REG_MINUTES = 0x01
    _REG_HOURS = 0x02
    _REG_DAY = 0x03
    _REG_DATE = 0x04
    _REG_MONTH = 0x05
    _REG_YEAR = 0x06
    _REG_CONTROL = 0x07

    def __init__(self, bus=0, address=0x68):
        self._bus = smbus_m.SMBus3(bus)
        self._addr = addr

    def _write(self, register, data):
        # print "addr =0x%x register = 0x%x data = 0x%x %i " % (self._addr, register, data,_bcd_to_int(data))
        self._bus.write_byte_data(self._addr, register, data)

    def _read(self, data):

        returndata = self._bus.read_byte_data(self._addr, data)
        # print "addr = 0x%x data = 0x%x %i returndata = 0x%x %i " % (self._addr, data, data, returndata, _bcd_to_int(returndata))
        return returndata

    def _read_seconds(self):
        return _bcd_to_int(self._read(self._REG_SECONDS))

    def _read_minutes(self):
        return _bcd_to_int(self._read(self._REG_MINUTES))

    def _read_hours(self):
        d = self._read(self._REG_HOURS)
        if (d == 0x64):
            d = 0x40
        return _bcd_to_int(d & 0x3F)

    def _read_day(self):
        return _bcd_to_int(self._read(self._REG_DAY))

    def _read_date(self):
        return _bcd_to_int(self._read(self._REG_DATE))

    def _read_month(self):
        return _bcd_to_int(self._read(self._REG_MONTH))

    def _read_year(self):
        return _bcd_to_int(self._read(self._REG_YEAR))

    def read_all(self):
        """Return a tuple such as (year, month, date, day, hours, minutes,
        seconds).
        """
        return (self._read_year(), self._read_month(), self._read_date(),
                self._read_day(), self._read_hours(), self._read_minutes(),
                self._read_seconds())

    def read_str(self):
        """Return a string such as 'YY-DD-MMTHH-MM-SS'.
        """
        return '%02d-%02d-%02dT%02d:%02d:%02d' % (self._read_year(),
                                                  self._read_month(), self._read_date(), self._read_hours(),
                                                  self._read_minutes(), self._read_seconds())

    def read_datetime(self, century=21, tzinfo=None):
        """Return the datetime.datetime object.
        """
        return datetime((century - 1) * 100 + self._read_year(),
                        self._read_month(), self._read_date(), self._read_hours(),
                        self._read_minutes(), self._read_seconds(), 0, tzinfo=tzinfo)

    def write_all(self, seconds=None, minutes=None, hours=None, day=None,
                  date=None, month=None, year=None, save_as_24h=True):
        """Direct write un-none value.
        Range: seconds [0,59], minutes [0,59], hours [0,23],
               day [0,7], date [1-31], month [1-12], year [0-99].
        """
        if seconds is not None:
            if seconds < 0 or seconds > 59:
                raise ValueError('Seconds is out of range [0,59].')
            self._write(self._REG_SECONDS, _int_to_bcd(seconds))

        if minutes is not None:
            if minutes < 0 or minutes > 59:
                raise ValueError('Minutes is out of range [0,59].')
            self._write(self._REG_MINUTES, _int_to_bcd(minutes))

        if hours is not None:
            if hours < 0 or hours > 23:
                raise ValueError('Hours is out of range [0,23].')
            # not | 0x40 as in the orignal code
            self._write(self._REG_HOURS, _int_to_bcd(hours))

        if year is not None:
            if year < 0 or year > 99:
                raise ValueError('Years is out of range [0,99].')
            self._write(self._REG_YEAR, _int_to_bcd(year))

        if month is not None:
            if month < 1 or month > 12:
                raise ValueError('Month is out of range [1,12].')
            self._write(self._REG_MONTH, _int_to_bcd(month))

        if date is not None:
            if date < 1 or date > 31:
                raise ValueError('Date is out of range [1,31].')
            self._write(self._REG_DATE, _int_to_bcd(date))

        if day is not None:
            if day < 1 or day > 7:
                raise ValueError('Day is out of range [1,7].')
            self._write(self._REG_DAY, _int_to_bcd(day))

    def write_datetime(self, dt):
        """Write from a datetime.datetime object.
        """
        self.write_all(dt.second, dt.minute, dt.hour,
                       dt.isoweekday(), dt.day, dt.month, dt.year % 100)

    def write_now(self):
        """Equal to DS1307.write_datetime(datetime.datetime.now()).
        """
        self.write_datetime(datetime.now())


if __name__ == "__main__":

    print "=========================================================="
    print "DS1307 and DS3231N demo for RPI series"
    print ""
    print "=========================================================="
    time.sleep(2)

    bus = 1
    addr = 0x68

    rtc = DS1307(bus, addr)
    rtc.write_now()

    # Main Loop - sleeps 10 seconds, then reads and prints values of all clocks
    try:
        while True:
            #currenttime = datetime.utcnow()

            #deltatime = currenttime - starttime

            print ""
            print "Raspberry Pi=\t" + time.strftime("%Y-%m-%d %H:%M:%S")
            print "DS1307=\t\t%s" % rtc.read_datetime()

            time.sleep(10.0)

    except KeyboardInterrupt:

        print("interruption")
        # Ctrl-C to exit
        sys.exit(0)



# **********************************************************************;
# * Project           : RPILib
# *
# * Program name      : L293.py
# *
# * Author            : Viacheslav Karpizin
# *
# * Date created      : 20200402
# *
# * Purpose           : Control both channels of L293 (L293D) motor driver chip
# *
# * Revision History  :
# *
# * Date        Author      Ref    Revision (Date in YYYYMMDD format)
# * 202004    Viacheslav Karpizin      1       original version
# *
#
# Copyright (c) 2020, Viacheslav Karpizin

#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.sparkfun.com/datasheets/Robotics/TB6612FNG.pdf
# **********************************************************************
import RPi.GPIO as GPIO
import time
import smbus_m
import sys
sys.path.append('./../')


GPIO.setmode(GPIO.BOARD)
# GPIO.setmode(GPIO.BCM)


class L293:
	ain1 = ""
	ain2 = ""
	bin1 = ""
 	bin2 = ""
	aen = ""
  ben = ""

	# Defaults
	freq = 200
	

	# Constructor
	def __init__(self, ain1, ain2, aen, bin1 = None, bin2 = None, ben = None):
		self.ain1 = ain1
		self.ain2 = ain2
		self.aen = aen
		self.bin1 = bin1
		self.bin2 = bin2
		self.ben = ben




		GPIO.setup(ain1,GPIO.OUT)
		GPIO.setup(ain2,GPIO.OUT)
		GPIO.setup(aen,GPIO.OUT)
		if (bin1 != None):
			GPIO.setup(bin1,GPIO.OUT)
		if bin2 != None:
			GPIO.setup(bin2,GPIO.OUT)
		if ben != None: 
			GPIO.setup(ben,GPIO.OUT)
                  
		if bin2 != None:
    			self.pb = GPIO.PWM(bin2, self.freq)
    			self.pb.start(0)


		self.pa = GPIO.PWM(ain2, self.freq)
		self.pa.start(0)

	# Speed from -100 to 100
	def drive(self,  speed, motor=1 ):
		# Negative speed for reverse, positive for forward
		# If necessary use reverse parameter in constructor
		dutyCycle = speed
		if(speed < 0):
			dutyCycle = dutyCycle * -1


    		if motor==1:
      			if(speed > 0):
				print("speed + for motor1")
        			GPIO.output(self.ain1,GPIO.HIGH)
        			self.pa.ChangeDutyCycle(dutyCycle)
      			else:
				print("speed - for motor1")
        			GPIO.output(self.ain1,GPIO.LOW)
        			self.pa.ChangeDutyCycle(100-dutyCycle)
      			
    		elif motor==2 and self.bin1 != None:
      			if(speed > 0):
        			GPIO.output(self.bin1,GPIO.HIGH)
        			self.pb.ChangeDutyCycle(dutyCycle)
      			else:
        			GPIO.output(self.bin1,GPIO.LOW)
        			self.pb.ChangeDutyCycle(100-dutyCycle)
    		else:
      			print("incorrect motor id")

	def brake(self, motor=1):
    		if motor==1:
			  self.pa.ChangeDutyCycle(0)
		  	  GPIO.output(self.ain1,GPIO.HIGH)

    		else:
			  self.pb.ChangeDutyCycle(0)
			  GPIO.output(self.bin1,GPIO.HIGH)
 

	def standby(self, value, motor=1):
		
    		if motor==1:
			GPIO.output(self.aen, value)
		
      		elif motor==2 and self.ben != None:
			GPIO.output(self.aen, value)
		else:
      			print("incorrect motor id")
			# raise exception


	def __del__(self):
		GPIO.cleanup()
    
    
    
def main():
	print "=========================================================="
	print "L293 deni for RPI Series "
	print "=========================================================="
	time.sleep(0.5)
	
	
	# test = L293 (a1, a2, aen, b1, b2, ben )
  	test = L293(15, 16, 13,None, None, None)

	
	

	try:
		while True:

			print("motor1, forward, 100%")
      			test.drive(100) #Forward 100% dutycycle
      			time.sleep(3)


					
                        print("motor1, forward, 20%")
                        test.drive(20) #Forward 100% dutycycle
                        time.sleep(3)

      			print("motor1, reverse, 100%")
      			test.drive(-100) #Backwards 100% dutycycle
      			time.sleep(3)

                        print("motor1, reverse, 10%")
                        test.drive(-10) #Backwards 100% dutycycle
                        time.sleep(3)


      
     	 		print("motor1, brake")
      			test.brake() #Short brake
      			time.sleep(1)

	                print("motor1, standby")

      			test.standby(True) #Enable standby
      			test.standby(False) #Disable standby   
			time.sleep(2)
			
	except KeyboardInterrupt:
		# Ctrl-C to exit
		sys.exit(0)


if __name__ =="__main__":
    main()   
    
    

